/*	ccmap.cc
	Crabclaw Map functions
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ccmap.h"
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <fstream>

#define RANDOM(h) ((int)((double)rand() / ((double)RAND_MAX + 1) * h));
#define RGB565PX(r,g,b) ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | (b >> 3)
#define RGB555PX(r,g,b) ((r & 0xF8) << 7) | ((g & 0xF8) << 2) | (b >> 3)
#define RGB888PX(r,g,b) (r << 16) | (g << 8) | (b)

using namespace std;

string __uitoa(unsigned int n)	{
	std::string buf = "";
	int num;

	if(n == 0)
		return "0";
	while(n > 0)	{
		num = n % 10;
		n /= 10;
		buf = (char)(num + '0') + buf;
	}
	return buf;
}

ccMap::ccMap()	{
	width_px = 0;
	height_px = 0;
	width_t = 1;
	height_t = 1;
	tile_width = 0;
	tile_height = 0;
	version = 0x00000902;
	current_layer = 0;
	back_layer = 0;
	colorkey = 0;
	layer_count = 1;
	ccLayer *ground = new ccLayer(this);
	layers.push_back(ground);
}

ccMap::~ccMap()	{
	std::vector<ccLayer*>::iterator it;
	for(it = layers.begin(); it != layers.end(); it++)	{
		delete (*it);
	}

	std::vector<ccTile*>::iterator i;
	for(i = tiles.begin(); i != tiles.end(); i++)	{
		delete (*i);
	}

	std::vector<ccAnim*>::iterator iter;
	for(iter = anims.begin(); iter != anims.end(); iter++)	{
		delete (*iter);
	}
}

void ccMap::draw(wxDC *context)	{
	if(layers.empty())
		return;

	if(back_layer != current_layer)
		layers[back_layer]->draw(context);
		
	layers[current_layer]->draw(context);
}

unsigned long ccMap::getWidth()	{
	return width_px;
}

unsigned long ccMap::getWidthTiles()	{
	return width_t;
}

unsigned long ccMap::getHeight()	{
	return height_px;
}

unsigned long ccMap::getHeightTiles()	{
	return height_t;
}

unsigned long ccMap::getTileWidth()	{
	return tile_width;
}

unsigned long ccMap::getTileHeight()	{
	return tile_height;
}

void ccMap::setWidth(unsigned long w)	{
	std::vector<ccLayer*>::iterator it;
	for(it = layers.begin(); it != layers.end(); it++)	{
		(*it)->setSize(w, height_t);
	}
	width_t = w;
	width_px = w * tile_width;
}

void ccMap::setHeight(unsigned long h)	{
	std::vector<ccLayer*>::iterator it;
	for(it = layers.begin(); it != layers.end(); it++)	{
		(*it)->setSize(width_t, h);
	}
	height_t = h;
	height_px = h * tile_height;
	layers[0]->randomizeLayer();
}

void ccMap::setTileAt(unsigned long x, unsigned long y, unsigned long t)	{
	layers[current_layer]->setTileNumber(x, y, t);
}

void ccMap::setTileHeight(unsigned short h)	{
	tile_height = h;
}

void ccMap::setTileWidth(unsigned short w)	{
	tile_width = w;
}

void ccMap::setVersion(unsigned short maj, unsigned char min, unsigned char mic)	{
	version = (maj << 16) | (min << 8) | (mic);
}

std::string ccMap::getVersionString()	{
	std::string str = "";
	str += __uitoa(version >> 16) + '.';
	str += __uitoa((version >> 8) & 0xFF) + '.';
	str += __uitoa(version & 0xFF);

	return str;
}

ccTile *ccMap::getTile(unsigned long num)	{
	if(num & 0x80000000)	{
		if((num & 0x7FFFFFFF) < anims.size())
			return tiles[anims[num & 0x7FFFFFFF]->getTile(0)];
		else
			return NULL;
	}
	else if(num < tiles.size())
		return tiles[num];
	else
		return NULL;
}

ccAnim *ccMap::getAnim(unsigned long num)	{
	if(num < anims.size())
		return anims[num];
	else
		return NULL;
}

void ccMap::loadTiles(const char *fn)	{
	wxImage *img = new wxImage(fn);
	wxImage *subImg;
	int w, h;
	int x, y;
	ccTile *tmp;

	if(img == NULL)
		return;
	w = img->GetWidth();
	h = img->GetHeight();

	for(y = 0; y < h; y += tile_height)	{
		for(x = 0; x < w; x += tile_width)	{
			subImg = new wxImage(img->GetSubImage(wxRect(x, y, tile_width,
			                                             tile_height)));
			tmp = new ccTile(this);
			tmp->setTexture(subImg, txrfmt);
			tiles.push_back(tmp);
		}
	}

	delete img;
}

void ccMap::addLayer(ccLayer *lay, bool inc)	{
	layers.push_back(lay);
	if(inc)	{
		layer_count++;
	}
}

void ccMap::addAnim(ccAnim *an, bool inc)	{
	anims.push_back(an);
	if(inc)	{
		animation_count++;
	}
}

unsigned long ccMap::getAnimCount()	{
	return animation_count;
}

unsigned long ccMap::getTileCount()	{
	return tiles.size();
}

static unsigned char __pow2(unsigned short num)	{
	unsigned char i;

	for(i = 0; i < 11; i++)	{
		if(pow((double)2, (double)i) == num)
			return i;
	}
	return 0;
}

static void __prepare_uint32(unsigned long num, unsigned char* array)	{
	array[3] = static_cast<unsigned char>((num & 0xFF000000) >> 24);
	array[2] = static_cast<unsigned char>((num & 0x00FF0000) >> 16);
	array[1] = static_cast<unsigned char>((num & 0x0000FF00) >> 8);
	array[0] = static_cast<unsigned char>((num & 0x000000FF));
}

static void __prepare_uint16(unsigned short num, unsigned char* array)	{
	array[1] = static_cast<unsigned char>((num & 0xFF00) >> 8);
	array[0] = static_cast<unsigned char>((num & 0x00FF));
}

static unsigned long __to_uint32(unsigned char* array)	{
	return array[0] | (array[1] << 8) | (array[2] << 16) | (array[3] << 24);
}

static unsigned short __to_uint16(unsigned char* array)	{
	return array[0] | (array[1] << 8);
}

void ccMap::save(std::string fn)	{
	ofstream out(fn.c_str(), ios::out | ios::trunc | ios::binary);
	char buf[4];
	unsigned long i, j, k;

	if(!out.is_open())	{
		wxMessageBox("Could not open map for output.", "Error", wxICON_ERROR);
		return;
	}

	out << "CCM";
	__prepare_uint32(version, (unsigned char *)buf);
	out.write(buf, 4);
	__prepare_uint32(width_t, (unsigned char *)buf);
	out.write(buf, 4);
	__prepare_uint32(height_t, (unsigned char *)buf);
	out.write(buf, 4);
	__prepare_uint32(layer_count, (unsigned char *)buf);
	out.write(buf, 4);
	out.put(__pow2(tile_width));
	out.put(__pow2(tile_height));
	__prepare_uint32(txrfmt, (unsigned char *)buf);
	out.write(buf, 4);
	__prepare_uint32((unsigned long)tiles.size(), (unsigned char *)buf);
	out.write(buf, 4);
	__prepare_uint32(anims.size(), (unsigned char *)buf);
	out.write(buf, 4);
	__prepare_uint32(colorkey, (unsigned char *)buf);
	out.write(buf, 4);

	for(i = 0; i < anims.size(); i++)	{
		out.put(anims[i]->getTileCount());
		__prepare_uint16(anims[i]->getDelay(), (unsigned char *)buf);
		out.write(buf, 2);
		for(j = 0; j < anims[i]->getTileCount(); j++)	{
			__prepare_uint32(anims[i]->getTile(j), (unsigned char *)buf);
			out.write(buf, 4);
		}
	}

	for(i = 0; i < layers.size(); i++)	{
		out.put(layers[i]->getScrollSpeed());
		for(j = 0; j < height_t; j++)	{
			for(k = 0; k < width_t; k++)	{
				__prepare_uint32(layers[i]->getTileNumber(k, j), (unsigned char *)buf);
				out.write(buf, 4);
			}
		}
	}

	for(i = 0; i < tiles.size(); i++)	{
		__prepare_uint32(tiles[i]->get32bitUserData(), (unsigned char *)buf);
		out.write(buf, 4);
		__prepare_uint16(tiles[i]->get16bitUserData(1), (unsigned char *)buf);
		out.write(buf, 2);
		__prepare_uint16(tiles[i]->get16bitUserData(2), (unsigned char *)buf);
		out.write(buf, 2);
		out.put(tiles[i]->get8bitUserData(1));
		out.put(tiles[i]->get8bitUserData(2));
		out.put(tiles[i]->get8bitUserData(3));
		out.put(tiles[i]->get8bitUserData(4));
		out.write((const char *)tiles[i]->getCollisionMap(),
		          tile_width * tile_height / 8);
		switch(txrfmt)	{
			case 1:
			case 3:
				out.write((const char *)(tiles[i]->getTexture()),
				          tile_width * tile_height * 2);
				break;
			case 2:
				out.write((const char *)(tiles[i]->getTexture()),
				          tile_width * tile_height * 3);
				break;
			default:
				cout << "Unsupported texture format: " << txrfmt;
				break;
		}
	}

	out.close();
}

ccMap *ccMap::load(std::string fn)	{
	ccMap *newmap = new ccMap();
	ifstream in(fn.c_str(), ios::in | ios::binary);
	char buf[4];
	unsigned long i, j, k, tile_count;
	ccTile *tmp;
	wxImage *tmp2;
	unsigned char *img;

	if(!in.is_open())	{
		wxMessageBox("Could not open map for input.", "Error", wxICON_ERROR);
		delete newmap;
		return NULL;
	}

	in.read(buf, 3);
	buf[3] = 0;

	if(strcmp(buf, "CCM"))	{
		wxMessageBox("Selected file is not a Crabclaw map.", "Error", 
		             wxICON_ERROR);
		delete newmap;
		return NULL;
	}

	in.read(buf, 4);
	newmap->version = __to_uint32((unsigned char *)buf);
	in.read(buf, 4);
	newmap->width_t = __to_uint32((unsigned char *)buf);
	in.read(buf, 4);
	newmap->height_t = __to_uint32((unsigned char *)buf);
	in.read(buf, 4);
	newmap->layer_count = __to_uint32((unsigned char *)buf);
	in.read(buf, 2);
	newmap->tile_width = (unsigned short)pow((double)2, (double)buf[0]);
	newmap->tile_height = (unsigned short)pow((double)2, (double)buf[1]);
	newmap->setWidth(newmap->width_t);
	newmap->setHeight(newmap->height_t);
	in.read(buf, 4);
	newmap->txrfmt = __to_uint32((unsigned char *)buf);
	in.read(buf, 4);
	tile_count = __to_uint32((unsigned char *)buf);
	in.read(buf, 4);
	newmap->animation_count = __to_uint32((unsigned char *)buf);
	in.read(buf, 4);
	newmap->colorkey = __to_uint32((unsigned char *)buf);

	for(i = 0; i < newmap->animation_count; i++)	{
		ccAnim *newanim = new ccAnim();
		in.read(buf, 1);
		k = buf[0];
		in.read(buf, 2);
		newanim->setDelay(__to_uint16((unsigned char *)buf));
		for(j = 0; j < k; j++)	{
			in.read(buf, 4);
			newanim->addTile(__to_uint32((unsigned char *)buf));
		}
		newmap->addAnim(newanim, false);
	}

	in.read(buf, 1);
	newmap->layers[0]->setScrollSpeed(buf[0]);
	for(i = 0; i < newmap->height_t; i++)	{
		for(j = 0; j < newmap->width_t; j++)	{
			in.read(buf, 4);
			newmap->layers[0]->setTileNumber(j, i, __to_uint32((unsigned char *)buf));
		}
	}

	for(i = 1; i < newmap->layer_count; i++)	{
		ccLayer *newlayer = new ccLayer(newmap);
		in.read(buf, 1);
		newlayer->setScrollSpeed(buf[0]);
		for(j = 0; j < newmap->height_t; j++)	{
			for(k = 0; k < newmap->width_t; k++)	{
				in.read(buf, 4);
				newlayer->setTileNumber(k, j, __to_uint32((unsigned char *)buf));
			}
		}
		newmap->addLayer(newlayer, false);
	}

	for(i = 0; i < tile_count; i++)	{
		tmp = new ccTile(newmap);
		in.read(buf, 4);
		tmp->set32bitUserData(__to_uint32((unsigned char *)buf));
		in.read(buf, 2);
		tmp->set16bitUserData(1, __to_uint16((unsigned char *)buf));
		in.read(buf, 2);
		tmp->set16bitUserData(2, __to_uint16((unsigned char *)buf));
		in.read(buf, 4);
		tmp->set8bitUserData(1, (unsigned char)buf[0]);
		tmp->set8bitUserData(2, (unsigned char)buf[1]);
		tmp->set8bitUserData(3, (unsigned char)buf[2]);
		tmp->set8bitUserData(4, (unsigned char)buf[3]);
		for(k = 0; k < newmap->tile_height; k++)	{
			for(j = 0; j < newmap->tile_width; j += 8)	{
				in.read(buf, 1);
				tmp->setPassable(j, k, buf[0] & 0x01);
				tmp->setPassable(j + 1, k, buf[0] & 0x02);
				tmp->setPassable(j + 2, k, buf[0] & 0x04);
				tmp->setPassable(j + 3, k, buf[0] & 0x08);
				tmp->setPassable(j + 4, k, buf[0] & 0x10);
				tmp->setPassable(j + 5, k, buf[0] & 0x20);
				tmp->setPassable(j + 6, k, buf[0] & 0x40);
				tmp->setPassable(j + 7, k, buf[0] & 0x80);
			}
		}
		img = (unsigned char *)malloc(newmap->tile_width * 
		                              newmap->tile_height * 3);
		switch(newmap->txrfmt)	{
			case 1:
				for(j = 0; (int)j < newmap->tile_width * newmap->tile_height; j++)	{
					in.read(buf, 2);
					img[j * 3] = (buf[1] & 0xF8);
					img[j * 3 + 1] = (((buf[1] & 0x07) << 3) | ((buf[0] & 0xFF) >> 5)) << 2;
					img[j * 3 + 2] = (buf[0] & 0x1F) << 3;
				}
				break;
			case 2:
				for(j = 0; (int)j < newmap->tile_width * newmap->tile_height; j++)	{
					in.read(buf, 3);
					img[j * 3] = buf[0];
					img[j * 3 + 1] = buf[1];
					img[j * 3 + 2] = buf[2];
				}
				break;
			case 3:
				for(j = 0; (int)j < newmap->tile_width * newmap->tile_height; j++)	{
					unsigned short s;

					in.read(buf, 2);
					s = __to_uint16((unsigned char *)buf);
					img[j * 3] = (s & 0x7C00) >> 7;
					img[j * 3 + 1] = (s & 0x3E0) >> 2;
					img[j * 3 + 2] = (s & 0x1F) << 3;
				}
				break;
		}

		tmp2 = new wxImage(newmap->tile_width, newmap->tile_height, img);
		tmp->setTexture(tmp2, newmap->txrfmt);
		tmp->setColorKey(newmap->colorkey);
		newmap->tiles.push_back(tmp);
	}

	in.close();

	return newmap;
}

void ccMap::setTextureFormat(unsigned long fmt)	{
	txrfmt = fmt;

	switch(fmt)	{
		case 1:
			colorkey &= 0xF8FCF8;
			break;
		case 3:
			colorkey &= 0xF8F8F8;
			break;
	}
}

unsigned long ccMap::getColorKey()	{
	return colorkey;
}

unsigned long ccMap::getLayerCount()	{
	return layer_count;
}

void ccMap::setCurrentLayer(unsigned long layer)	{
	current_layer = layer;
}

void ccMap::setBackLayer(unsigned long layer)	{
	back_layer = layer;
}

void ccMap::setColorKey(unsigned long ck)	{
	vector<ccTile*>::iterator it;

	switch(txrfmt)	{
		case 1:
			ck &= 0xF8FCF8;
			break;
		case 3:
			ck &= 0xF8F8F8;
			break;
	}

	if(ck == colorkey)
		return;

	colorkey = ck;

	for(it = tiles.begin(); it != tiles.end(); it++)	{
		(*it)->setColorKey(ck);
	}
}

void ccMap::exportToPNG(const char *fn)	{
	wxMemoryDC png_out;
	wxBitmap bmp(width_px, height_px, 24);
	vector<ccLayer*>::iterator i;

	if(layers.empty())
		return;

	png_out.SelectObject(bmp);

	for(i = layers.begin(); i != layers.end(); i++)	{
		(*i)->draw(&png_out);
	}

	png_out.SelectObject(wxNullBitmap);

	bmp.SaveFile(fn, wxBITMAP_TYPE_PNG);
}

ccLayer::ccLayer(ccMap *own)	{
	owner = own;
	map = (unsigned long **) malloc(sizeof(unsigned long *) *
	                                own->getHeightTiles());
	for(unsigned long i = 0; i < own->getHeightTiles(); i++)	{
		map[i] = (unsigned long *) malloc(4 * own->getWidthTiles());
		memset(map[i], 0, 4 * own->getWidthTiles());
	}
}

ccLayer::~ccLayer()	{
	for(unsigned long i = 0; i < owner->getHeightTiles(); i++)	{
		free(map[i]);
	}
	free(map);
}

void ccLayer::setSize(unsigned long x, unsigned long y)	{
	map = (unsigned long **) realloc(map, sizeof(unsigned long *) * y);
	for(unsigned long i = 0; i < y; i++)	{
		if(y < owner->getHeightTiles())
			map[i] = (unsigned long *) realloc(map[i], 4 * x);
		else	{
			map[i] = (unsigned long *) malloc(4 * x);
			memset(map[i], 0, 4 * x);
		}
	}
}

void ccLayer::setScrollSpeed(unsigned char sc)	{
	scroll = sc;
}

void ccLayer::setTileNumber(unsigned long x, unsigned long y, unsigned long t)	{
	map[y][x] = t;
}

unsigned char ccLayer::getScrollSpeed()	{
	return scroll;
}

unsigned long ccLayer::getTileNumber(unsigned long x, unsigned long y)	{
	if(x < owner->getWidthTiles())
		if(y < owner->getHeightTiles())
			return map[y][x];

	return 0;
}

void ccLayer::draw(wxDC *context)	{
	for(unsigned long i = 0; i < owner->getHeightTiles(); i++)	{
		for(unsigned long j = 0; j < owner->getWidthTiles(); j++)	{
			owner->getTile(map[i][j])->draw(context, j, i);
		}
	}
}

void ccLayer::randomizeLayer()	{
	for(unsigned long i = 0; i < owner->getHeightTiles(); i++)	{
		for(unsigned long j = 0; j < owner->getWidthTiles(); j++)	{
			map[i][j] = RANDOM(owner->getTileCount());
		}
	}
}

ccTile::ccTile(ccMap *ownr)	{
	owner = ownr;
	user1 = 0;
	user2 = user3 = 0;
	user4 = user5 = user6 = user7 = 0;
	collision_map = (unsigned char *) malloc(ownr->getTileWidth() *
	                                         ownr->getTileHeight() / 8);
	memset(collision_map, 1, ownr->getTileWidth() * ownr->getTileHeight() / 8);
	img = NULL;
}

ccTile::~ccTile()	{
	delete img;
	free(txr);
	free(collision_map);
}

void ccTile::set32bitUserData(unsigned long val)	{
	user1 = val;
}

void ccTile::set16bitUserData(unsigned char num, unsigned short val)	{
	switch(num)	{
		case 1:
			user2 = val;
			break;
		case 2:
			user3 = val;
			break;
		default:
			cout << "Attempt to set invalid 16-bit user data" << endl;
	}
}

void ccTile::set8bitUserData(unsigned char num, unsigned char val)	{
	switch(num)	{
		case 1:
			user4 = val;
			break;
		case 2:
			user5 = val;
			break;
		case 3:
			user6 = val;
			break;
		case 4:
			user7 = val;
			break;
		default:
			cout << "Attempt to set invalid 8-bit user data" << endl;
	}
}

void ccTile::setPassable(unsigned short x, unsigned short y, bool pass)	{
	int num = y * owner->getTileWidth() + x;
	if(pass)
		collision_map[num / 8] |= (1 << (num % 8));
	else
		collision_map[num / 8] &= ~(1 << (num % 8));
}

void ccTile::setTexture(wxImage *im, unsigned long txrfmt)	{
	int i;
	img = im;
	unsigned char *data = img->GetData();

	switch(txrfmt)	{
		case 1:
			txr = (unsigned char *) malloc(2 * img->GetWidth() * img->GetHeight());
			for(i = 0; i < img->GetWidth() * img->GetHeight(); i++)	{
				data[i * 3] &= 0xF8;
				data[i * 3 + 1] &= 0xFC;
				data[i * 3 + 2] &= 0xF8;
				txr[i * 2 + 1] = (data[i * 3]) | (data[i * 3 + 1] >> 5);
				txr[i * 2] = (data[i * 3 + 1] << 3) | (data[i * 3 + 2] >> 3);
			}
			break;
		case 2:
			txr = img->GetData();
			break;
		case 3:
			txr = (unsigned char *) malloc(2 * img->GetWidth() * img->GetHeight());
			for(i = 0; i < img->GetWidth() * img->GetHeight(); i++)	{
				data[i * 3] &= 0xF8;
				data[i * 3 + 1] &= 0xF8;
				data[i * 3 + 2] &= 0xF8;
				txr[i * 2 + 1] = (data[i * 3] >> 1) | (data[i * 3 + 1] >> 6);
				txr[i * 2] = (data[i * 3 + 1] << 2) | (data[i * 3 + 2] >> 3);
			}
			break;
		default:
			cout << "Unsupported Texture format: " << hex << txrfmt << dec 
			     << endl;
			break;
	}
	bmp = wxBitmap(img, -1);
}

bool ccTile::isPassable(unsigned short x, unsigned short y)	{
	int num = y * owner->getTileWidth() + x;
	return (collision_map[num / 8] & (1 << (num % 8)));
}

unsigned char *ccTile::getCollisionMap()	{
	return collision_map;
}

unsigned long ccTile::get32bitUserData()	{
	return user1;
}

unsigned short ccTile::get16bitUserData(unsigned char num)	{
	switch(num)	{
		case 1:
			return user2;
		case 2:
			return user3;
		default:
			cout << "Attempt to read invalid 16-bit user data" << endl;
			return 0;
	}
}

unsigned char ccTile::get8bitUserData(unsigned char num)	{
	switch(num)	{
		case 1:
			return user4;
		case 2:
			return user5;
		case 3:
			return user6;
		case 4:
			return user7;
		default:
			cout << "Attempt to read invalid 8-bit user data" << endl;
			return 0;
	}
}

unsigned char *ccTile::getTexture()	{
	return txr;
}

void ccTile::draw(wxDC *context, unsigned long x, unsigned long y)	{
	context->DrawBitmap(bmp, x * owner->getTileWidth(), y *
	                    owner->getTileHeight(), true);
}

void ccTile::setColorKey(unsigned long ck)	{
	wxColour col;

	col.Set((ck >> 16) & 0xFF, (ck >> 8) & 0xFF, (ck) & 0xFF);

	bmp.SetMask(new wxMask(bmp, col));
}

wxBitmap ccTile::getBitmap()	{
	return bmp;
}

ccAnim::ccAnim()	{
	delay = 100;
}

ccAnim::~ccAnim()	{
}

void ccAnim::setDelay(unsigned short d)	{
	delay = d;
}

void ccAnim::addTile(unsigned long t)	{
	tiles.push_back(t);
}

void ccAnim::removeTile(unsigned char num)	{
	vector<unsigned long>::iterator i;
	unsigned int it = 0;

	for(i = tiles.begin(); i != tiles.end(); i++)	{
		if(it == num)	{
			tiles.erase(i);
			break;
		}
		it++;
	}
}

unsigned short ccAnim::getDelay()	{
	return delay;
}

unsigned long ccAnim::getTile(int n)	{
	return tiles[n];
}

unsigned char ccAnim::getTileCount()	{
	return (unsigned char) tiles.size();
}
